@@ -1,18 +1,17 @@
 import collections
-import copy
 import contextlib
+import copy
 import os
 import time
 import warnings
 
 import torch
 
+from pytorch_pfn_extras import writing
+from pytorch_pfn_extras.reporting import Reporter
 from pytorch_pfn_extras.training import extension as extension_module
 from pytorch_pfn_extras.training import trigger as trigger_module
 from pytorch_pfn_extras.training import util as util_module
-from pytorch_pfn_extras.reporting import Reporter
-from pytorch_pfn_extras import writing
-
 
 # Select the best-resolution timer function
 try:
@@ -67,7 +66,8 @@ class _BaseExtensionsManager:
             self._stop_trigger = trigger_module.get_trigger(
                 (max_epochs, 'epoch'))
         else:
-            self._stop_trigger = stop_trigger
+            self._stop_trigger = trigger_module.get_trigger(
+                stop_trigger)
         if writer is None:
             writer = writing.SimpleWriter(out_dir=out_dir)
         # triggers are stateful, so we need to make a copy for internal use
@@ -76,6 +76,7 @@ class _BaseExtensionsManager:
         self._out = out_dir
         self.writer = writer
         self.reporter = Reporter()
+        self._start_extensions_called = False
 
         if not isinstance(models, dict):
             if not isinstance(models, torch.nn.Module):
@@ -106,24 +107,51 @@ class _BaseExtensionsManager:
         # Initialize the writer
         self.writer.initialize(self.out)
 
+    # All properties cannot be accessed without starting extensions, because
+    # the snapshot extension must run first to restore the training state.
+
+    @property
+    def iteration(self):
+        self.start_extensions()
+        return self._iteration
+
+    @iteration.setter
+    def iteration(self, value):
+        self._iteration = value
+
+    @property
+    def models(self):
+        self.start_extensions()
+        return self._models
+
+    @property
+    def optimizers(self):
+        self.start_extensions()
+        return self._optimizers
+
     @property
     def elapsed_time(self):
+        # Unavailable until the initial run_iteration call
         return _get_time()-self._start_time
 
     @property
     def is_before_training(self):
+        # Extensions will start via self.iteration
         return self.iteration == 0
 
     @property
     def epoch(self):
+        # Extensions will start via self.iteration
         return self.iteration // self._iters_per_epoch
 
     @property
     def epoch_detail(self):
+        # Extensions will start via self.iteration
         return self.iteration / self._iters_per_epoch
 
     @property
     def stop_trigger(self):
+        self.start_extensions()
         # Trigger is stateful, we close the extensions the first time
         # it evaluates to True, as it won't do it again
         return self._stop_trigger(self)
@@ -152,6 +180,12 @@ class _BaseExtensionsManager:
         self._iters_per_epoch = iters_per_epoch
 
     def start_extensions(self):
+        if self._start_extensions_called:
+            # This method must not be called twice or more.
+            return
+        else:
+            self._start_extensions_called = True
+
         exts = self._extensions
         extension_order = sorted(
             exts.keys(),
@@ -299,19 +333,21 @@ class _BaseExtensionsManager:
     def load_state_dict(self, to_load, *, transform_models=lambda n, x: x):
         """
         transform_models is a function that apply a transformation
-        to a model.
+        to a model before loading its state
 
         When using a `torch.nn.DataParallel` model, if we want
-        to load a model with the `torch.nn.DataParallel` applied
+        to load the original state in a model with the
+        `torch.nn.DataParallel` applied
         load_state_dict(
-            state, transform_models=lambda n, x: torch.nn.DataParallel(x))
+            state, transform_models=(lambda n, x:
+                x.module if isinstance(x, torch.nn.DataParallel) else  x))
         """
         self._start_iteration = to_load['_start_iteration']
         self.iteration = self._start_iteration
         for name in self._models:
             # TODO(ecastill) map_loc when loading the model and DDP check
-            self._models[name].load_state_dict(to_load['models'][name])
-            self._models[name] = transform_models(name, self._models[name])
+            transformed_model = transform_models(name, self._models[name])
+            transformed_model.load_state_dict(to_load['models'][name])
 
         for name in self._optimizers:
             self._optimizers[name].load_state_dict(to_load['optimizers'][name])
@@ -329,7 +365,8 @@ class ExtensionsManager(_BaseExtensionsManager):
             or an actual Module
         optimizers (dict or `torch.Optimizer`): Map of string to Optimizer
             or an actual Optimizer.
-        max_epochs (int): Number of epochs in the whole training loop.
+        max_epochs (int): Number of epochs in the whole training loop. Ignored
+            if `stop_trigger` is passed as a kwarg.
         iters_per_epoch (int): Number of iterations in one epoch.
         extensions (list or None): List of Extentions to be used.
         out_dir (str): Output directory (default: ``result``).
@@ -444,12 +481,15 @@ class IgniteExtensionsManager(_BaseExtensionsManager):
 
         @self.engine.on(Events.STARTED)
         def set_training_started(engine):
+            iters_per_epoch = len(engine.state.dataloader)
+            # Initialize manager once before extensions' `initialize` call
+            self._prepare_for_training(0, iters_per_epoch)
             self.start_extensions()
             start_iteration = self._start_iteration
             self.engine.state.iteration = self._start_iteration
             self.engine.state.epoch = self._start_epoch
             self._start_time = _get_time()
-            iters_per_epoch = len(engine.state.dataloader)
+            # Initialize manager again after all state is restored
             self._prepare_for_training(start_iteration, iters_per_epoch)
 
             # Make all the next
